#include <gint/display.h>

#include "vec.h"

extern inline void vec_cpy(Vec *destination, Vec source) {
	/* memcpy for vectors */
	destination->x = source.x;
	destination->y = source.y;
}

extern inline void vec_add(Vec *vector, Vec force) {
	/* addition */
	vector->x += force.x;
	vector->y += force.y;
}

extern inline void vec_sub(Vec *vector, Vec force) {
	/* substraction */
	vector->x -= force.x;
	vector->y -= force.y;
}

extern inline void vec_mul(Vec *vector, int scale) {
	/* multiplication */
	vector->x *= scale;
	vector->y *= scale;
}

extern inline void vec_mulf(Vec *vector, float scale) {
	/* floating point multiplication */
	vector->x *= scale;
	vector->y *= scale;
}

extern inline void vec_div(Vec *vector, int scale) {
	/* division */
	vector->x /= scale;
	vector->y /= scale;
}

extern inline void vec_divf(Vec *vector, float scale) {
	/* floating point division */
	vector->x /= scale;
	vector->y /= scale;
}

extern inline void vec_lerp(Vec *from, Vec to, float scale) {
	/* A linear interpolation function, can be used for camera and
	 * animations. 'scale' is the transformation speed, 1 being
	 * instant. */
	 from->x = from->x * (1 - scale) + to.x * scale;
	 from->y = from->y * (1 - scale) + to.y * scale;
}

extern inline void vec_clamp(Vec *to_limit, Vec min, Vec max) {
	/* clamp a vector between two points */
	if (to_limit->x < min.x) to_limit->x = min.x;
	if (to_limit->y < min.y) to_limit->y = min.y;
	if (to_limit->x > max.x) to_limit->x = max.x;
	if (to_limit->y > max.y) to_limit->y = max.y;
}

extern inline void vec_drect(Vec top_left, Vec bottom_right, int color) {
	/* Draw a rectangle corresponding to two vectors position with
	 * given 'color'.
	 */
	drect(top_left.x, top_left.y, bottom_right.x, bottom_right.y, color);
}
